Package: moanin
Title: An R Package for Time Course RNASeq Data Analysis
Version: 1.18.0
Authors@R: c(person("Elizabeth", "Purdom", email="epurdom@stat.berkeley.edu",
                    role=c("aut"),
                    comment=c("orcid"="https://orcid.org/0000-0001-9455-7990")),
             person("Nelle", "Varoquaux", role=c("aut", "cre"),
                    email="nelle.varoquaux@gmail.com",
                    comment=c(orcid="https://orcid.org/0000-0002-8748-6546")))
Description: Simple and efficient workflow for time-course gene expression
	     data, built on publictly available open-source projects hosted on
	     CRAN and bioconductor. moanin provides helper functions for all
	     the steps required for analysing time-course data using
	     functional data analysis: (1) functional modeling of the
	     timecourse data; (2) differential expression analysis; (3)
	     clustering; (4) downstream analysis.
Depends: R (>= 4.0), SummarizedExperiment, topGO, stats
Imports: S4Vectors, MASS (>= 1.0.0), limma, viridis, edgeR, graphics,
        methods, grDevices, reshape2, NMI, zoo, ClusterR, splines,
        matrixStats
Suggests: testthat (>= 1.0.0), timecoursedata, knitr, rmarkdown,
        markdown, covr, BiocStyle
VignetteBuilder: knitr
License: BSD 3-clause License + file LICENSE
Encoding: UTF-8
LazyData: false
RoxygenNote: 7.1.1
biocViews: TimeCourse, GeneExpression, RNASeq, Microarray,
        DifferentialExpression, Clustering
git_url: https://git.bioconductor.org/packages/moanin
git_branch: RELEASE_3_22
git_last_commit: 6b72807
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:10:47 UTC; biocbuild
Author: Elizabeth Purdom [aut] (ORCID: <https://orcid.org/0000-0001-9455-7990>),
  Nelle Varoquaux [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8748-6546>)
Maintainer: Nelle Varoquaux <nelle.varoquaux@gmail.com>
