# Generated by roxygen2: do not edit by hand

export(consensus_matrix)
export(create_go_term_mapping)
export(discont_basis)
export(find_enriched_go_terms)
export(get_auc_similarity_scores)
export(perWeek_barplot)
export(plot_cdf_consensus)
export(plot_model_explorer)
export(pvalues_fisher_method)
exportClasses(Moanin)
exportMethods("[")
exportMethods("group_variable<-")
exportMethods("group_variable_name<-")
exportMethods("time_variable<-")
exportMethods("time_variable_name<-")
exportMethods(DE_timecourse)
exportMethods(DE_timepoints)
exportMethods(basis_matrix)
exportMethods(create_moanin_model)
exportMethods(create_timepoints_contrasts)
exportMethods(degrees_of_freedom)
exportMethods(estimate_log_fold_change)
exportMethods(get_log_data)
exportMethods(group_variable)
exportMethods(group_variable_name)
exportMethods(log_transform)
exportMethods(plot_splines_data)
exportMethods(rescale_values)
exportMethods(show)
exportMethods(spline_formula)
exportMethods(splines_kmeans)
exportMethods(splines_kmeans_predict)
exportMethods(splines_kmeans_score_and_label)
exportMethods(time_by_group_variable)
exportMethods(time_variable)
exportMethods(time_variable_name)
import(SummarizedExperiment)
import(methods)
importClassesFrom(S4Vectors,DataFrame)
importFrom(ClusterR,KMeans_rcpp)
importFrom(edgeR,DGEList)
importFrom(edgeR,calcNormFactors)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,plot.new)
importFrom(graphics,plot.window)
importFrom(limma,contrasts.fit)
importFrom(limma,eBayes)
importFrom(limma,lmFit)
importFrom(limma,voom)
importFrom(methods,new)
importFrom(splines,ns)
importFrom(stats,as.formula)
importFrom(topGO,GenTable)
importFrom(topGO,annFUN.gene2GO)
importFrom(topGO,runTest)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(viridis,viridis)
